/*
 * Copyright (c) 2000 Peter Pentchev
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "rname.h"
#include "rn_err.h"

static char cvs_id[] __UNUS = "$Id: rname.c,v 1.3 2000/11/10 14:31:44 roam Exp $";

char	rn_verstr[64] = "rname";

/***
Function:
        rn_makeversion()                - generate version string
Inputs:
        none
Returns:
        rn_err_t
Modifies:
        rn_verstr
***/

static rn_err_t
rn_makeversion(void) {

  snprintf(rn_verstr, sizeof(rn_verstr), "rname v%d.%d"
#if RN_VER_PRE
      "-pre%d"
#endif /* RN_VER_PRE */
#if RN_VER_PATCH
      "p%d"
#endif /* RN_VER_PATCH */
      ,
      RN_VER_MAJ, RN_VER_MIN
#if RN_VER_PRE
      , RN_VER_PRE
#endif /* RN_VER_PRE */
#if RN_VER_PATCH
      , RN_VER_PATCH
#endif /* RN_VER_PATCH */
      );

  return RN_ERR_NONE;
}

/***
Function:
	rn_init				- general-purpose init function
Inputs:
	argc, argv			- main() args for option processing
Returns:
	rn_err_t
	CMDLINE
Modifies:
	verbose, quiet
***/

rn_err_t
rn_init(int argc __UNUS, char **argv __UNUS) {
  int ch, helpq = 0, versq = 0;

  rn_makeversion();

  /* Process cmdline options */
  
  opterr = 0;
  while(ch = getopt(argc, argv, RN_OPTSTR), ch != EOF)
    switch(ch) {
      case 'h':
	helpq = 1;
	break;
      case 'V':
	versq = 1;
	break;
      case 'v':
	verbose++;
	break;
      case 'q':
	quiet++;
	break;
      case '?':
      default:
	return RN_ERR_CMDLINE;
    }

  if (helpq) {
    rn_version();
    rn_help();
    exit(RN_ERR_NONE);
  } else if (versq) {
    rn_version();
    exit(RN_ERR_NONE);
  }
  
  argc -= optind; argv += optind;
  if (argc < 2) {
    rn_help();
    exit(RN_ERR_CMDLINE);
  }

  return RN_ERR_NONE;
}

/***
Function:
	rn_help					- startup help info
Inputs:
	none
Returns:
	RN_ERR_NONE
Modifies:
	nothing, writes to stdout
***/

rn_err_t
rn_help(void) {
  
  RINF(("%s",
	"Usage: rname [-hqvV] fakename realname [args..]\n"
	"\t-h\tprint this help text and exit;\n"
	"\t-v\tverbose operation; multiple -v's increase verbosity level;\n"
	"\t-q\tquiet operation; multiple -q's make it even more quiet;\n"
	"\t-V\tprint version information and exit.\n"
       ));
  return RN_ERR_NONE;
}

/***
function:
	rn_version				- output version info
Inputs:
	none
Returns:
	RN_ERR_NONE
Modifies:
	nothing, writes to stdout
***/

rn_err_t
rn_version(void) {

  RINF(("%s\n", rn_verstr));

  if (verbose) {
    RINF(("%s\n", "Built on " __DATE__ ", " __TIME__));
#ifdef __GNUC__
    if (verbose > 1)
      RINF(("%s\n", "Compiler: GNU C " __VERSION__ " on " RN_OS " " RN_OSREL
	    ": " RN_OSHOST));
#endif /* __GNUC__ */
  }
  return RN_ERR_NONE;
}

/***
Function:
	rn_doit				- actually execute something
Inputs:
	none
Returns:
	rn_err_t
	EXEC				- execv()/execvp() failed
Modifies:
	overlays argv[1] with argv[0], then exec's the requested program
***/

static rn_err_t
rn_doit(int argc __UNUS, char **argv) {
  char *realname;

  /* save the progname to be executed, then place the fake name in its place */
  realname = argv[1];
  argv[1] = argv[0];

  /* if there is no / in the program name, search PATH */
  if (strchr(realname, '/') == NULL)
    execvp(realname, argv+1);
  else
    execv(realname, argv+1);

  /* uhh.. we shouldn't be here, should we now? */
  return RN_ERR_EXEC;
}

/***
  M A I N   F U N C T I O N
***/

int
rn_main(int argc, char **argv) {
  rn_err_t r;

  if (r = rn_init(argc, argv), r)
    return rn_prerror("init", r);

  argc -= optind; argv += optind;
  if (r = rn_doit(argc, argv), r)
    return rn_prerror("doit", r);
  
  return RN_ERR_NONE;
}
